import { format as d3Format } from 'd3-format';
import { SupportedLocales } from '@cloudflare/intl-core';
import moment from 'moment';
const DEFAULT_LOCALE = SupportedLocales.en_US;
const DEFAULT_CURRENCY = 'USD';
const SI_PATTERN = /^([^a-zA-Z]*)([a-zA-Z]?$)/;
/**
 * Moment formatters per-locale examples.
 *
 *   en-US
 * -------------------
 * LongDate:            September 4, 1986
 * ShortDate:           Sep 4, 1986
 * Time:                1:30 PM
 * DateTime:            September 4, 1986
 * ISO8601:             1986-09-04T13:30:00-07:00
 * LongDayMonth:        4 September
 * ShortDayMonth:       4 Sep
 *
 *   de-DE
 * -------------------
 * LongDate:            4. September 1986
 * ShortDate:           4. Sep. 1986
 * Time:                13:30
 * DateTime:            4. September 1986 13:30
 * ISO8601:             1986-09-04T13:30:00-07:00
 * LongDayMonth:        4 September
 * ShortDayMonth:       4 Sep.
 */

export let DateFormatters;

(function (DateFormatters) {
  DateFormatters["LongDate"] = "LL";
  DateFormatters["ShortDate"] = "ll";
  DateFormatters["Time"] = "LT";
  DateFormatters["DateTime"] = "LLL";
  DateFormatters["ISO8601"] = "YYYY-MM-DDTHH:mm:ssZ";
  DateFormatters["LongDayMonth"] = "D MMMM";
  DateFormatters["ShortDayMonth"] = "D MMM";
  DateFormatters["DateTimeWithSeparators"] = "L LT";
  DateFormatters["DateTimeSecondsWithSeparators"] = "L LTS";
})(DateFormatters || (DateFormatters = {}));

export const formatDate = (value, formatter, utc = false) => {
  // TODO: Implement global UTC pyarreference fetching and formatting
  let formattedDate = moment(value);

  if (utc) {
    formattedDate = formattedDate.utc();
  }

  return formattedDate.format(formatter);
};
export const localizeNumberWithPrecision = (number, locale = DEFAULT_LOCALE) => {
  return number.toLocaleString(locale, {
    maximumFractionDigits: 20
  });
};
export const formatNumber = (number, locale = DEFAULT_LOCALE, abbreviate = false, decimalPlaces = 2, trimInsignificantZeros = true, addSpaceAfterNumber = false) => {
  let val = number;
  let siSuffix = ''; // SI units = International system of units.
  // Don't use SI units for sub-1 numbers (milli, micro, etc)

  if (abbreviate && number >= 1) {
    // d3Format notes:
    // 1.) s refers to decimal notation with an SI prefix, rounded to significant digits.
    // Example input and output: d3.format("s")(1500);  // "1.50000k".
    // 2.) The exec() method on a regular expression executes a search for a match in a
    // specified string. Returns a result array, or null.
    const siMatch = SI_PATTERN.exec(d3Format('s')(number)); // Split value from SI unit.
    // siMatch is an array that looks like:
    // ["479.247M", "479.247", "M", index: 0, input: "479.247M", groups: undefined]

    if (siMatch) {
      val = +siMatch[1];
      siSuffix = siMatch[2] || '';
    }
  } // d3Format notes:
  // 1.) ~ trims insignificant trailing digits.
  // 2.) f refers to fixed point notation.


  const formatted = d3Format(`.${decimalPlaces}${trimInsignificantZeros ? '~' : ''}f`)(val);
  const formattedAsLocale = Number(formatted).toLocaleString(locale);
  const siToShortScaleSuffix = {
    k: 'k',
    M: 'M',
    G: 'B',
    T: 't',
    P: 'q'
  }; // Note - abbreviate can be set to either `true`, which will abbreviate the number
  // as 1.5B for 1.5 billion. Or, it can be set to the string 'si', which will format
  // the number using the SI suffix instead. For billions this is 'giga' which makes
  // sense for bits and bytes, but does not for formatting counts of say, requests.

  const out = abbreviate === true && siToShortScaleSuffix.hasOwnProperty(siSuffix) ? [formattedAsLocale, siToShortScaleSuffix[siSuffix]] : [formattedAsLocale, siSuffix];
  return out.join(addSpaceAfterNumber ? ' ' : '');
};
export const formatCurrency = (value = 0, options = {
  locale: DEFAULT_LOCALE
}, minimumFractionDigits = 2, maximumFractionDigits = 2) => {
  return Number(value).toLocaleString(options.locale, {
    style: 'currency',
    // We should *always* display the currency that was billed in (USD).
    currency: DEFAULT_CURRENCY,
    minimumFractionDigits,
    maximumFractionDigits
  })
  /**
   * `Number.toLocaleString` with `style: 'currency'` behaves differently
   * across vendors/browsers. On top of this, the spec/dcoumentation is
   * mysterious on how it _should_ work. Read more here—
   * https://stackoverflow.com/a/56161892.
   *
   * tl;dr
   * -------------------
   * When formatting in `USD` currency with a non-`en-US` locale,
   * `toLocaleString` will format the string like so: `400,00 $`. Looks
   * fine, but the weird part is that the space is actually a unicode
   * 160 char (non-breaking space), not a regular unicode 30 char (normal
   * space). Furthermore, our Trans/DOMPurify implementation will try to
   * sanitize this string and renders `400,00&nbsp;$` to the DOM.
   *
   * So.. to fix this, we simply replace any abnormal "space" unicode chars
   * with a regular space. The regex comes from—
   * https://stackoverflow.com/a/51399843.
   */
  .replace(/[\u202F\u00A0\u2000\u2001\u2003]/, ' ');
};
/**
 * @param {Number} number
 * @param {String} locale
 * @param {Boolean} [includeDecimals]
 * @return {String}
 */

export const formatBytes = (number, locale = DEFAULT_LOCALE, includeDecimals) => {
  return formatNumber(number, locale, 'si', includeDecimals ? 2 : 0) // Add space between value and SI unit
  .replace(/([a-zA-Z]?)$/, ' $1') + 'B';
};
/**
 * @param {Number} number
 * @param {String} locale
 * @param {Boolean} [includeDecimals]
 * @param {Boolean} [useBit]
 * @return {String}
 */

export const formatBits = (number, locale = DEFAULT_LOCALE, includeDecimals, useBit = true) => {
  return formatNumber(number, locale, 'si', includeDecimals ? 2 : 0) // Add space between value and SI unit
  .replace(/([a-zA-Z]?)$/, ' $1') + `${useBit ? 'bit' : 'b'}`;
};
/**
 * @param {Number} number
 * @param {String} locale
 * @param {Boolean} [useSI]
 * @param {Number} [decimalPlaces]
 * @param {Boolean} [trimInsignificantZeros]
 * @return {String}
 */

export const formatNumberForAnalytics = (number, locale = DEFAULT_LOCALE, useSI = false, decimalPlaces = 2, trimInsignificantZeros = true) => {
  return formatNumber(number, locale, useSI, decimalPlaces, trimInsignificantZeros);
};
export const capitalizeStr = str => str && str.charAt(0).toUpperCase() + str.slice(1);
export const formatPercentage = (value, total) => `${Math.round(value * 100 / total * 100) / 100}%`;
export const roundPercentage = (val, roundDecimalCases = 0, locale = DEFAULT_LOCALE) => {
  const roundDiv = Math.pow(10, roundDecimalCases); // Treat ~0% and ~100% values specially so the precision is surfaced

  let localizedString = '';

  if (val > 99) {
    // Edge case: between 99 and 100, produce values with precision like this:
    // 99.999735 --> 99.9997
    localizedString = localizeNumberWithPrecision(100 - +(100 - val).toPrecision(1), locale);
  } else if (val < 1) {
    // Edge case: between 0 and 1, produce values with the precision of one
    // significant digit:
    // 0.000273 --> 0.0003
    localizedString = localizeNumberWithPrecision(+val.toPrecision(1), locale);
  } else {
    // In all other cases, round to the nearest integer:
    // 78.612 --> 79
    localizedString = formatNumberForAnalytics(val * roundDiv / roundDiv, locale, false, roundDecimalCases);
  }

  return localizedString + '%';
};